% =========================================================
% SK-Hype algorithm
% ---------------------------------------------------------
% Related paper: 
% 
% J. CHEN, C. RICHARD and P. HONEINE, "Nonlinear unmixing of hyperspectral
% data based on a linear-mixture/nonlinear-fluctuation model". IEEE Transactions on signal processing, 2012. 
%
% Optimal step-size is searched by the golden segmentation  
% ---------------------------------------------------------
% Utilization recommendation:
% 1. Parameters (Kernel bandwidth/regularization parameter) should be tuned according to scenes.
% 2. Here we use the function "qpas" for the optimization instead of the general optimization 
%    function of matlab: "quadprog", as the later is rather slow. One can
%    replace all "qpas" by "quadprog", or other optimization functions in case
%    there's a problem of running.
%    The full toolbox QuadprogC is also attached for respecting its
%    completeness. 
%
% Version: 13-sept-2012
%
% Jie CHEN & Cedric Richard
% chen@unice.fr, cedric.richard@unice.fr
% Laboratoire Lagrange, Universit de Nice Sophia-antipolis
% Nice, France


clear


% ============  Parameters to tune ======================== 
% Gaussian kernel bandwidth : 
par = 2;
% Regualrization parameter : 
% \mu in the paper = 1/C
C = 100;

% ===========Parameters for step-size searching ==========
GR = (sqrt(5)+1)/2-1;
MU = 1;
GEPS = 0.01;
GEPSM = 0.01;


% ============== Scene Parameters =========================
%  (see the function generate_image for detailes)
% endmember database (only 3 availabe in this demo due to the endmembers.mat)
Material_database = 3;
% number of endmember (maximum to 8 in this demo due to the endmembers.mat)
R = 5; 
% mixture model (1: linear, 2: bilinear, 3:postnonlinear)
Mixture_model=3;
% number of pixels to generate
N = 10;
% snr
SNR = 30;

%  ============== Image generation  =======================
[r,M,a,noise_std]=generate_image(Material_database, R, Mixture_model, N, SNR);
[L, R] = size(M);



% ============= Gaussian kernel calculation =================
Q=eye(R)/par^2;
MQM=M*Q*M';
dMQM = diag(MQM);
KM = exp(-0.5*(dMQM*ones(L,1)'+ones(L,1)*dMQM'-2*M*Q*M'));
% For using the polynomial proposed polynomial kernel, remove the
% comment symbol:
% KM = (1+1/R^2*(M-0.5)*(M-0.5)').^2;


MM =M*M';


d = [0.5;0.5];
dd1(1)=d(1);
dd2(1)=d(2);
iter_total=0;
ttt=0;

v=0.0;
tic

for n = 1 : N
    if mod(n,10)==0, n, end
    y = r(:,n);
    d = [0.5;0.5];
    K = d(1)*MM+d(2)*KM;
    Kt = [K+1/C*eye(L)];
    H = [Kt,    d(1)*M;
         d(1)*M',  d(1)*eye(R)];        
    H=(H+H')/2+0e-5*eye(L+R);
    f = -[y;zeros(R,1)];
    A = -[zeros(R,L),eye(R)];
    b = zeros(R,1);    

    z0 = qpas(H,f,A,b);
    
    z = z0(1:L);
    gam = z0(L+1:L+R);
    
    ht = (M'*z+gam)*d(1);
    y_rc = M*ht+d(2)*KM*z;
    cost = ht'*ht/d(1)+d(2)*z'*KM*z+C*(y-y_rc)'*(y-y_rc);
    cost_old=cost;
    z_p=z;
    
    for iter = 1 : 15                % Maximum iteration 15
 
        %============ Gradient of d  =============
        dJ_d = -(1/2*ht'*ht/d(1)^2)+1/2*z'*KM*z;
         
        %============= Golden Search =============
        %------------- Initialization ------------
        if dJ_d>=0
        m1= d(1)/dJ_d;
        m2= 0;
        else 
        m1= (d(1)-1)/dJ_d(1);
        m2= 0;
        end
        
        mu_max=min(m1, MU);
        mu_min=max(m2,0);
        d_max(1) = d(1) - mu_max*dJ_d(1);
        K_max = d_max(1)*MM+(1-d_max(1))*KM;
        Kt = [K_max+1/C*eye(L)];
        H = [Kt,    d_max(1) *M;
            d_max(1) *M',  d_max(1) *eye(R)];        
        H=(H+H')/2+0e-5*eye(L+R);
        z0 = qpas(H,f,A,b); 
        z_max = z0(1:L);
        gam_max = z0(L+1:L+R); 
        ht = (M'*z_max+gam_max)*d_max(1);
        y_rc = M*ht+(1-d_max(1))*KM*z_max;
        cost_max =ht'*ht/d_max(1)+(1-d_max(1))*z_max'*KM*z_max+C*(y-y_rc)'*(y-y_rc);
             
        cost_min = cost;
        mus = (mu_min+mu_max)/2*ones(1,4);
        while (mu_max-mu_min)>GEPS & mu_max > GEPSM;

            mu_l=mu_min+(mu_max-mu_min)*(1-GR);
            mu_r=mu_l+(mu_max-mu_l)*(1-GR);
            %-------------Propotions------------------
            d_l(1) = d(1) - mu_l*dJ_d(1);
            d_l(2) = 1 - d_l(1);
            d_r(1) = d(1) - mu_r*dJ_d(1);
            d_r(2) = 1 - d_r(1);
            %---------------Kernels-------------------
            K_l = d_l(1)*MM+d_l(2)*KM;
            K_r = d_r(1)*MM+d_r(2)*KM;
            %--------------Solving--------------------
            Kt = [K_l+1/C*eye(L)];
            H = [Kt,   d_l(1)*M;
                d_l(1)* M',  d_l(1)*eye(R)];        
            H=(H+H')/2+0e-5*eye(L+R);
            z0 = qpas(H,f,A,b); 
            z_l = z0(1:L);
            gam_l = z0(L+1:L+R); 
            ht = (M'*z_l+gam_l)*d_l(1);
            y_rc = M*ht+(1-d_l(1))*KM*z_l;
            cost_l =ht'*ht/d_l(1)+(1-d_l(1))*z_l'*KM*z_l+C*(y-y_rc)'*(y-y_rc);
 
            Kt = [K_r+1/C*eye(L)];
            H = [Kt,    d_r(1)* M;
                 d_r(1)* M',   d_r(1)*eye(R)];        
            H=(H+H')/2+0e-5*eye(L+R);
            z0 = qpas(H,f,A,b); 
            z_r = z0(1:L);
            gam_r = z0(L+1:L+R); 
            ht = (M'*z_r+gam_r)*d_r(1);
            y_rc = M*ht+(1-d_r(1))*KM*z_r;
            cost_r = ht'*ht/d_r(1)+(1-d_r(1))*z_r'*KM*z_r+C*(y-y_rc)'*(y-y_rc);
            %------------------cases-------------------
            mus = [mu_min mu_l mu_r mu_max];
            costs = [cost_min cost_l cost_r cost_max];
            [val,idx] = min(costs);
            switch idx
                case 1
                    mu_max = mu_l;
                    cost_max = cost_l;
                    z = z_l;
                    gam = gam_l;
                case 2
                    mu_max = mu_r;
                    cost_max = cost_r;
                    z = z_r;
                    gam = gam_r;                    
                case 3
                    mu_min = mu_l;
                    cost_min = cost_l;
                    z = z_l;
                    gam = gam_l;                    
                case 4
                    mu_min = mu_r;
                    cost_min = cost_r;
                    z = z_r;
                    gam = gam_r; 
            end
        end

        %=============== Final Updta ================
        mu = mus(idx);
        cost =val;
    %    if cost <= cost_old;
            d(1) = d(1) - mu * dJ_d;
            d(2) = 1 - d(1);
    %    end;

        ht =  (M'*z+gam)*d(1);
        
        dd1(iter+1)=d(1);
        dd2(iter+1)=d(2); 
        erry_inst(iter) = cost;

       % if (iter >1 & abs(erry_inst(iter)-erry_inst(iter-1))/erry_inst(iter-1)< 1e-4)
       %     break;
       % end

        if (iter >1 & norm(d-dp)/R< 1e-4)      % Stop criterion
             break;
        end
        dp=d;

    end

     beta=z;

     h = d(1)*(M'*beta+gam);
     a_est(:,n)=max(h/sum(h),0);
      
 %    r_rec(:,n) = d(1)*M*h+d(2)*KM*beta;
end
toc
[RMSE, std] = ErrComput(a(:,1:N),a_est)
%err_rec(nn) = erry/N;

%err=sqrt(norm(a_est(:,1:N)-a(:,1:N),'fro')^2/(R*N));

